@@ -56,6 +56,7 @@ from gajim.common import app
 from gajim.common import gpg
 from gajim.common import passwords
 from gajim.common import idle
+from gajim.common import i18n
 from gajim.common.i18n import _
 from gajim.common.connection_handlers import *
 from gajim.common.contacts import GC_Contact
@@ -904,10 +905,6 @@ class Connection(CommonConnection, ConnectionHandlers):
         ]
         self._hostname = hostname
 
-        if h:
-            app.resolver.resolve('_xmppconnect.' + helpers.idn_to_ascii(h),
-                                 self._on_resolve_txt, type_='txt')
-
         if use_srv and self._proxy is None:
             self._srv_hosts = []
 
@@ -917,6 +914,9 @@ class Connection(CommonConnection, ConnectionHandlers):
             for service in services:
                 record_name = '_' + service + '._tcp.' + helpers.idn_to_ascii(h)
                 app.resolver.resolve(record_name, self._on_resolve_srv)
+
+            app.resolver.resolve('_xmppconnect.' + helpers.idn_to_ascii(h),
+                                 self._on_resolve_txt, type_='txt')
         else:
             self._connect_to_next_host()
 
@@ -1037,7 +1037,8 @@ class Connection(CommonConnection, ConnectionHandlers):
         con = nbxmpp.NonBlockingClient(
             domain=self._hostname,
             caller=self,
-            idlequeue=app.idlequeue)
+            idlequeue=app.idlequeue,
+            lang=i18n.LANG)
 
         # increase default timeout for server responses
         nbxmpp.dispatcher_nb.DEFAULT_TIMEOUT_SECONDS = \
