@@ -1,42 +1,40 @@
-# -*- coding:utf-8 -*-
-## src/common/contacts.py
-##
-## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
-##                    Travis Shirk <travis AT pobox.com>
-##                    Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
-##                         Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
-##                    Tomasz Melcer <liori AT exroot.org>
-##                    Julien Pivotto <roidelapluie AT gmail.com>
-## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+#                    Travis Shirk <travis AT pobox.com>
+#                    Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
+#                         Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+#                    Tomasz Melcer <liori AT exroot.org>
+#                    Julien Pivotto <roidelapluie AT gmail.com>
+# Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 try:
     from gajim.common import caps_cache
+    from gajim.common.i18n import _
     from gajim.common.account import Account
     from gajim import common
+    from gajim.common.const import Chatstate
 except ImportError as e:
     if __name__ != "__main__":
         raise ImportError(str(e))
 
-class XMPPEntity(object):
+class XMPPEntity:
     """
     Base representation of entities in XMPP
     """
@@ -49,7 +47,7 @@ class XMPPEntity(object):
 class CommonContact(XMPPEntity):
 
     def __init__(self, jid, account, resource, show, status, name,
-    our_chatstate, chatstate, client_caps=None):
+                 chatstate, client_caps=None):
 
         XMPPEntity.__init__(self, jid, account, resource)
 
@@ -59,11 +57,45 @@ class CommonContact(XMPPEntity):
 
         self.client_caps = client_caps or caps_cache.NullClientCaps()
 
-        # please read xep-85 http://www.xmpp.org/extensions/xep-0085.html
-        # this holds what WE SEND to contact (our current chatstate)
-        self.our_chatstate = our_chatstate
         # this is contact's chatstate
-        self.chatstate = chatstate
+        self._chatstate = chatstate
+
+        self._is_pm_contact = False
+
+    @property
+    def show(self):
+        return self._show
+
+    @show.setter
+    def show(self, value):
+        if not isinstance(value, str):
+            raise TypeError('show must be a string')
+        self._show = value
+
+    @property
+    def chatstate_enum(self):
+        return self._chatstate
+
+    @property
+    def chatstate(self):
+        if self._chatstate is None:
+            return
+        return str(self._chatstate)
+
+    @chatstate.setter
+    def chatstate(self, value):
+        if value is None:
+            self._chatstate = value
+        else:
+            self._chatstate = Chatstate[value.upper()]
+
+    @property
+    def is_gc_contact(self):
+        return isinstance(self, GC_Contact)
+
+    @property
+    def is_pm_contact(self):
+        return self._is_pm_contact
 
     def get_full_jid(self):
         raise NotImplementedError
@@ -82,8 +114,11 @@ class CommonContact(XMPPEntity):
             # show, so we can be sure it's existant. Otherwise, we still
             # return caps for a contact that has no resources left.
             return False
-        else:
-            return caps_cache.client_supports(self.client_caps, requested_feature)
+        return caps_cache.client_supports(self.client_caps, requested_feature)
+
+    @property
+    def uses_phone(self):
+        return caps_cache.get_client_identity(self.client_caps) == 'phone'
 
 
 class Contact(CommonContact):
@@ -92,18 +127,21 @@ class Contact(CommonContact):
     """
     def __init__(self, jid, account, name='', groups=None, show='', status='',
     sub='', ask='', resource='', priority=0, keyID='', client_caps=None,
-    our_chatstate=None, chatstate=None, idle_time=None, avatar_sha=None):
+    chatstate=None, idle_time=None, avatar_sha=None, groupchat=False,
+    is_pm_contact=False):
         if not isinstance(jid, str):
             print('no str')
         if groups is None:
             groups = []
 
         CommonContact.__init__(self, jid, account, resource, show, status, name,
-            our_chatstate, chatstate, client_caps=client_caps)
+                               chatstate, client_caps=client_caps)
 
         self.contact_name = '' # nick choosen by contact
         self.groups = [i if i else _('General') for i in set(groups)] # filter duplicate values
         self.avatar_sha = avatar_sha
+        self._is_groupchat = groupchat
+        self._is_pm_contact = is_pm_contact
 
         self.sub = sub
         self.ask = ask
@@ -129,14 +167,13 @@ class Contact(CommonContact):
     def get_shown_groups(self):
         if self.is_observer():
             return [_('Observers')]
-        elif self.is_groupchat():
+        if self.is_groupchat():
             return [_('Groupchats')]
-        elif self.is_transport():
+        if self.is_transport():
             return [_('Transports')]
-        elif not self.groups:
+        if not self.groups:
             return [_('General')]
-        else:
-            return self.groups
+        return self.groups
 
     def is_hidden_from_roster(self):
         """
@@ -149,7 +186,7 @@ class Contact(CommonContact):
             return False
         if self.sub in ('none', 'from') and self.ask == 'subscribe':
             return False
-        if self.sub in ('none', 'from') and (self.name or len(self.groups)):
+        if self.sub in ('none', 'from') and (self.name or self.groups):
             return False
         if _('Not in Roster') in self.groups:
             return False
@@ -164,10 +201,7 @@ class Contact(CommonContact):
         return is_observer
 
     def is_groupchat(self):
-        for account in common.app.gc_connected:
-            if self.jid in common.app.gc_connected[account]:
-                return True
-        return False
+        return self._is_groupchat
 
     def is_transport(self):
         # if not '@' or '@' starts the jid then contact is transport
@@ -180,11 +214,10 @@ class GC_Contact(CommonContact):
     """
 
     def __init__(self, room_jid, account, name='', show='', status='', role='',
-    affiliation='', jid='', resource='', our_chatstate=None,
-    chatstate=None, avatar_sha=None):
+    affiliation='', jid='', resource='', chatstate=None, avatar_sha=None):
 
         CommonContact.__init__(self, jid, account, resource, show, status, name,
-            our_chatstate, chatstate)
+                               chatstate)
 
         self.room_jid = room_jid
         self.role = role
@@ -206,7 +239,8 @@ class GC_Contact(CommonContact):
         """
         return Contact(jid=self.get_full_jid(), account=self.account,
             name=self.name, groups=[], show=self.show, status=self.status,
-            sub='none', client_caps=self.client_caps, avatar_sha=self.avatar_sha)
+            sub='none', client_caps=self.client_caps, avatar_sha=self.avatar_sha,
+            is_pm_contact=True)
 
 
 class LegacyContactsAPI:
@@ -239,8 +273,12 @@ class LegacyContactsAPI:
                 GC_Contacts())
         self._metacontact_manager.add_account(account_name)
 
-    def get_accounts(self):
-        return list(self._accounts.keys())
+    def get_accounts(self, zeroconf=True):
+        accounts = list(self._accounts.keys())
+        if not zeroconf:
+            if 'Local' in accounts:
+                accounts.remove('Local')
+        return accounts
 
     def remove_account(self, account):
         del self._accounts[account]
@@ -248,8 +286,8 @@ class LegacyContactsAPI:
 
     def create_contact(self, jid, account, name='', groups=None, show='',
     status='', sub='', ask='', resource='', priority=0, keyID='',
-    client_caps=None, our_chatstate=None, chatstate=None, idle_time=None,
-    avatar_sha=None):
+    client_caps=None, chatstate=None, idle_time=None,
+    avatar_sha=None, groupchat=False):
         if groups is None:
             groups = []
         # Use Account object if available
@@ -257,8 +295,8 @@ class LegacyContactsAPI:
         return Contact(jid=jid, account=account, name=name, groups=groups,
             show=show, status=status, sub=sub, ask=ask, resource=resource,
             priority=priority, keyID=keyID, client_caps=client_caps,
-            our_chatstate=our_chatstate, chatstate=chatstate,
-            idle_time=idle_time, avatar_sha=avatar_sha)
+            chatstate=chatstate, idle_time=idle_time, avatar_sha=avatar_sha,
+            groupchat=groupchat)
 
     def create_self_contact(self, jid, account, resource, show, status, priority,
     name='', keyID=''):
@@ -286,7 +324,7 @@ class LegacyContactsAPI:
             status=contact.status, sub=contact.sub, ask=contact.ask,
             resource=contact.resource, priority=contact.priority,
             keyID=contact.keyID, client_caps=contact.client_caps,
-            our_chatstate=contact.our_chatstate, chatstate=contact.chatstate,
+            chatstate=contact.chatstate_enum,
             idle_time=contact.idle_time, avatar_sha=contact.avatar_sha)
 
     def add_contact(self, account, contact):
@@ -304,6 +342,9 @@ class LegacyContactsAPI:
         if remove_meta:
             self._metacontact_manager.remove_metacontact(account, jid)
 
+    def get_groupchat_contact(self, account, jid):
+        return self._accounts[account].contacts.get_groupchat_contact(jid)
+
     def get_contacts(self, account, jid):
         return self._accounts[account].contacts.get_contacts(jid)
 
@@ -374,8 +415,8 @@ class LegacyContactsAPI:
             for jid in self.get_jid_list(account):
                 if jid == our_jid:
                     continue
-                if common.app.jid_is_transport(jid) and not \
-                _('Transports') in groups:
+                if (common.app.jid_is_transport(jid) and
+                        _('Transports') not in groups):
                     # do not count transports
                     continue
                 if self.has_brother(account, jid, accounts) and not \
@@ -404,8 +445,7 @@ class LegacyContactsAPI:
         # Only called if self has no attr_name
         if hasattr(self._metacontact_manager, attr_name):
             return getattr(self._metacontact_manager, attr_name)
-        else:
-            raise AttributeError(attr_name)
+        raise AttributeError(attr_name)
 
     def create_gc_contact(self, room_jid, account, name='', show='', status='',
             role='', affiliation='', jid='', resource='', avatar_sha=None):
@@ -443,6 +483,9 @@ class LegacyContactsAPI:
             return
         contact.avatar_sha = sha
 
+    def get_combined_chatstate(self, account, jid):
+        return self._accounts[account].contacts.get_combined_chatstate(jid)
+
 
 class Contacts():
     """
@@ -454,13 +497,13 @@ class Contacts():
         self._contacts = {}
 
     def add_contact(self, contact):
-        if contact.jid not in self._contacts:
+        if contact.jid not in self._contacts or contact.is_groupchat():
             self._contacts[contact.jid] = [contact]
             return
         contacts = self._contacts[contact.jid]
         # We had only one that was offline, remove it
         if len(contacts) == 1 and contacts[0].show == 'offline':
-            # Do not use self.remove_contact: it deteles
+            # Do not use self.remove_contact: it deletes
             # self._contacts[account][contact.jid]
             contacts.remove(contacts[0])
         # If same JID with same resource already exists, use the new one
@@ -475,7 +518,7 @@ class Contacts():
             return
         if contact in self._contacts[contact.jid]:
             self._contacts[contact.jid].remove(contact)
-        if len(self._contacts[contact.jid]) == 0:
+        if not self._contacts[contact.jid]:
             del self._contacts[contact.jid]
 
     def remove_jid(self, jid):
@@ -518,6 +561,12 @@ class Contacts():
                 if c.resource == resource:
                     return c
 
+    def get_groupchat_contact(self, jid):
+        if jid in self._contacts:
+            contacts = self._contacts[jid]
+            if contacts[0].is_groupchat():
+                return contacts[0]
+
     def get_avatar(self, jid, size=None, scale=None):
         if jid not in self._contacts:
             return None
@@ -589,6 +638,18 @@ class Contacts():
             self._contacts[new_jid].append(_contact)
             del self._contacts[old_jid]
 
+    def get_combined_chatstate(self, jid):
+        if jid not in self._contacts:
+            return
+        contacts = self._contacts[jid]
+        states = []
+        for contact in contacts:
+            if contact.chatstate_enum is None:
+                continue
+            states.append(contact.chatstate_enum)
+
+        return str(min(states)) if states else None
+
 
 class GC_Contacts():
 
@@ -609,7 +670,7 @@ class GC_Contacts():
             return
         del self._rooms[gc_contact.room_jid][gc_contact.name]
         # It was the last nick in room ?
-        if not len(self._rooms[gc_contact.room_jid]):
+        if not self._rooms[gc_contact.room_jid]:
             del self._rooms[gc_contact.room_jid]
 
     def remove_room(self, room_jid):
@@ -626,10 +687,10 @@ class GC_Contacts():
         return list(self._rooms[room_jid].keys())
 
     def get_gc_contact(self, room_jid, nick):
-        nick_list = self.get_nick_list(room_jid)
-        if not nick in nick_list:
+        try:
+            return self._rooms[room_jid][nick]
+        except KeyError:
             return None
-        return self._rooms[room_jid][nick]
 
     def is_gc_contact(self, jid):
         """
@@ -644,7 +705,7 @@ class GC_Contacts():
         if len(jid) != 2:
             return False
         gcc = self.get_gc_contact(jid[0], jid[1])
-        return gcc != None
+        return gcc is not None
 
     def get_nb_role_total_gc_contacts(self, room_jid, role):
         """
@@ -711,8 +772,9 @@ class MetacontactManager():
             self._metacontacts_tags[brother_account][tag] = [{'jid': brother_jid,
                     'tag': tag}]
             if brother_account != account:
-                common.app.connections[brother_account].store_metacontacts(
-                        self._metacontacts_tags[brother_account])
+                con = common.app.connections[brother_account]
+                con.get_module('MetaContacts').store_metacontacts(
+                    self._metacontacts_tags[brother_account])
         # be sure jid has no other tag
         old_tag = self._get_metacontacts_tag(account, jid)
         while old_tag:
@@ -727,11 +789,12 @@ class MetacontactManager():
             else:
                 self._metacontacts_tags[account][tag].append({'jid': jid,
                         'tag': tag})
-        common.app.connections[account].store_metacontacts(
-                self._metacontacts_tags[account])
+        con = common.app.connections[account]
+        con.get_module('MetaContacts').store_metacontacts(
+            self._metacontacts_tags[account])
 
     def remove_metacontact(self, account, jid):
-        if not account in self._metacontacts_tags:
+        if account not in self._metacontacts_tags:
             return
 
         found = None
@@ -742,8 +805,9 @@ class MetacontactManager():
                     break
             if found:
                 self._metacontacts_tags[account][tag].remove(found)
-                common.app.connections[account].store_metacontacts(
-                        self._metacontacts_tags[account])
+                con = common.app.connections[account]
+                con.get_module('MetaContacts').store_metacontacts(
+                    self._metacontacts_tags[account])
                 break
 
     def has_brother(self, account, jid, accounts):
@@ -789,11 +853,10 @@ class MetacontactManager():
         if not tag:
             return []
         answers = []
-        for account in self._metacontacts_tags:
-            if tag in self._metacontacts_tags[account]:
-                for data in self._metacontacts_tags[account][tag]:
-                    data['account'] = account
-                    answers.append(data)
+        if tag in self._metacontacts_tags[account]:
+            for data in self._metacontacts_tags[account][tag]:
+                data['account'] = account
+                answers.append(data)
         return answers
 
     def _metacontact_key(self, data):
@@ -847,7 +910,7 @@ class MetacontactManager():
 
     def _get_metacontacts_big_brother(self, family):
         """
-        Which of the family will be the big brother under wich all others will be
+        Which of the family will be the big brother under which all others will be
         ?
         """
         return max(family, key=self._metacontact_key)
