@@ -1,39 +1,36 @@
-## src/common/gpg.py
-##
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2005 Alex Mauer <hawke AT hawkesnest.net>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Jean-Marie Traissard <jim AT lapin.org>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2005 Alex Mauer <hawke AT hawkesnest.net>
+# Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Jean-Marie Traissard <jim AT lapin.org>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import os
 import logging
 from gajim.common import app
 
-if app.HAVE_GPG:
+if app.is_installed('GPG'):
     import gnupg
     gnupg.logger = logging.getLogger('gajim.c.gnupg')
 
     class GnuPG(gnupg.GPG):
         def __init__(self):
             use_agent = app.config.get('use_gpg_agent')
-            gnupg.GPG.__init__(self, gpgbinary=app.GPG_BINARY, use_agent=use_agent)
+            gnupg.GPG.__init__(self, gpgbinary=app.get_gpg_binary(), use_agent=use_agent)
             encoding = app.config.get('pgp_encoding')
             if encoding:
                 self.encoding = encoding
@@ -56,8 +53,7 @@ if app.HAVE_GPG:
                     if key['trust'] not in ('f', 'u'):
                         if key['keyid'][-8:] not in self.always_trust:
                             return '', 'NOT_TRUSTED ' + key['keyid'][-8:]
-                        else:
-                            trust = True
+                        trust = True
             result = super(GnuPG, self).encrypt(str_.encode('utf8'), recipients,
                 always_trust=trust, passphrase=self.passphrase)
 
@@ -73,7 +69,7 @@ if app.HAVE_GPG:
             result = super(GnuPG, self).decrypt(data.encode('utf8'),
                 passphrase=self.passphrase)
 
-            return str(result)
+            return result.data.decode('utf8')
 
         def sign(self, str_, keyID):
             result = super(GnuPG, self).sign(str_.encode('utf8'), keyid=keyID, detach=True,
@@ -124,7 +120,8 @@ if app.HAVE_GPG:
             """
             Remove header and footer from data
             """
-            if not data: return ''
+            if not data:
+                return ''
             lines = data.splitlines()
             while lines[0] != '':
                 lines.remove(lines[0])
@@ -133,7 +130,8 @@ if app.HAVE_GPG:
             i = 0
             for line in lines:
                 if line:
-                    if line[0] == '-': break
+                    if line[0] == '-':
+                        break
                 i = i+1
             line = '\n'.join(lines[0:i])
             return line
