@@ -40,48 +40,29 @@ import socket
 import time
 import logging
 import json
+import shutil
+import collections
+from io import StringIO
 from datetime import datetime, timedelta
 from distutils.version import LooseVersion as V
 from encodings.punycode import punycode_encode
 from string import Template
 
 import nbxmpp
+from nbxmpp.stringprepare import nameprep
+import precis_i18n.codec  # pylint: disable=unused-import
 
+from gajim.common import app
 from gajim.common import caps_cache
 from gajim.common import configpaths
 from gajim.common.i18n import Q_
 from gajim.common.i18n import _
 from gajim.common.i18n import ngettext
-from gajim.common.caps_cache import muc_caps_cache
+from gajim.common.const import Display
 
-try:
-    import precis_i18n.codec  # pylint: disable=unused-import
-    HAS_PRECIS_I18N = True
-except ImportError:
-    HAS_PRECIS_I18N = False
+if app.is_installed('PYCURL'):
+    import pycurl
 
-HAS_SOUND = True
-if sys.platform == 'win32':
-    try:
-        import winsound  # windows-only built-in module for playing wav
-    except ImportError:
-        HAS_SOUND = False
-        print('Gajim is not able to playback sound because'
-              'pywin32 is missing', file=sys.stderr)
-
-elif sys.platform == 'darwin':
-    try:
-        from AppKit import NSSound
-    except ImportError:
-        HAS_SOUND = False
-        print('Gajim is not able to playback sound because'
-              'pyobjc is missing', file=sys.stderr)
-
-try:
-    import wave     # posix-only fallback wav playback
-    import ossaudiodev as oss
-except Exception:
-    pass
 
 log = logging.getLogger('gajim.c.helpers')
 
@@ -180,10 +161,7 @@ def parse_resource(resource):
     """
     if resource:
         try:
-            if HAS_PRECIS_I18N:
-                return resource.encode('Nickname').decode('utf-8')
-            from nbxmpp.stringprepare import resourceprep
-            return resourceprep.prepare(resource)
+            return resource.encode('OpaqueString').decode('utf-8')
         except UnicodeError:
             raise InvalidFormat('Invalid character in resource.')
 
@@ -217,7 +195,6 @@ def prep(user, server, resource):
             if not server or len(server.encode('utf-8')) > 1023:
                 raise InvalidFormat(_('Server must be between 1 and 1023 bytes'))
             try:
-                from nbxmpp.stringprepare import nameprep
                 server = nameprep.prepare(server)
             except UnicodeError:
                 raise InvalidFormat(_('Invalid character in hostname.'))
@@ -228,11 +205,7 @@ def prep(user, server, resource):
         if not user or len(user.encode('utf-8')) > 1023:
             raise InvalidFormat(_('Username must be between 1 and 1023 bytes'))
         try:
-            if HAS_PRECIS_I18N:
-                user = user.encode('UsernameCaseMapped').decode('utf-8')
-            else:
-                from nbxmpp.stringprepare import nodeprep
-                user = nodeprep.prepare(user)
+            user = user.encode('UsernameCaseMapped').decode('utf-8')
         except UnicodeError:
             raise InvalidFormat(_('Invalid character in username.'))
     else:
@@ -242,11 +215,7 @@ def prep(user, server, resource):
         if not resource or len(resource.encode('utf-8')) > 1023:
             raise InvalidFormat(_('Resource must be between 1 and 1023 bytes'))
         try:
-            if HAS_PRECIS_I18N:
-                resource = resource.encode('OpaqueString').decode('utf-8')
-            else:
-                from nbxmpp.stringprepare import resourceprep
-                resource = resourceprep.prepare(resource)
+            resource = resource.encode('OpaqueString').decode('utf-8')
         except UnicodeError:
             raise InvalidFormat(_('Invalid character in resource.'))
     else:
@@ -444,22 +413,26 @@ def get_uf_chatstate(chatstate):
         return _('has closed the chat window or tab')
     return ''
 
-def is_in_path(command, return_abs_path=False):
-    """
-    Return True if 'command' is found in one of the directories in the user's
-    path. If 'return_abs_path' is True, return the absolute path of the first
-    found command instead. Return False otherwise and on errors
-    """
-    for directory in os.getenv('PATH').split(os.pathsep):
-        try:
-            if command in os.listdir(directory):
-                if return_abs_path:
-                    return os.path.join(directory, command)
-                return True
-        except OSError:
-            # If the user has non directories in his path
-            pass
-    return False
+def find_soundplayer():
+    if sys.platform in ('win32', 'darwin'):
+        return
+
+    if app.config.get('soundplayer') != '':
+        return
+
+    commands = ('aucat', 'paplay', 'aplay', 'play', 'ossplay')
+    for command in commands:
+        if shutil.which(command) is not None:
+            if command == 'paplay':
+                command += ' -n gajim --property=media.role=event'
+            elif command in ('aplay', 'play'):
+                command += ' -q'
+            elif command == 'ossplay':
+                command += ' -qq'
+            elif command == 'aucat':
+                command += ' -i'
+            app.config.set('soundplayer', command)
+            break
 
 def exec_command(command, use_shell=False, posix=True):
     """
@@ -586,12 +559,6 @@ def datetime_tuple(timestamp):
         tim = tim.timetuple()
     return tim
 
-
-from gajim.common import app
-if app.is_installed('PYCURL'):
-    import pycurl
-    from io import StringIO
-
 def convert_bytes(string):
     suffix = ''
     # IEC standard says KiB = 1024 bytes KB = 1000 bytes
@@ -758,38 +725,52 @@ def strip_soundfile_path(file_, dirs=None, abs_=True):
     return file_
 
 def play_sound_file(path_to_soundfile):
-    if path_to_soundfile == 'beep':
-        exec_command('beep')
-        return
     path_to_soundfile = check_soundfile_path(path_to_soundfile)
     if path_to_soundfile is None:
         return
-    if sys.platform == 'win32' and HAS_SOUND:
+
+    if sys.platform == 'win32':
+        import winsound
         try:
             winsound.PlaySound(path_to_soundfile,
-                    winsound.SND_FILENAME|winsound.SND_ASYNC)
+                               winsound.SND_FILENAME|winsound.SND_ASYNC)
         except Exception:
             log.exception('Sound Playback Error')
-    elif sys.platform == 'linux':
-        if app.config.get('soundplayer') == '':
-            def _oss_play():
-                sndfile = wave.open(path_to_soundfile, 'rb')
-                nc, sw, fr, nf, _comptype, _compname = sndfile.getparams()
-                dev = oss.open('/dev/dsp', 'w')
-                dev.setparameters(sw * 8, nc, fr)
-                dev.write(sndfile.readframes(nf))
-                sndfile.close()
-                dev.close()
-            app.thread_interface(_oss_play)
+
+    elif sys.platform == 'darwin':
+        try:
+            from AppKit import NSSound
+        except ImportError:
+            log.exception('Sound Playback Error')
             return
-        player = app.config.get('soundplayer')
-        command = build_command(player, path_to_soundfile)
-        exec_command(command)
-    elif sys.platform == 'darwin' and HAS_SOUND:
+
         sound = NSSound.alloc()
         sound.initWithContentsOfFile_byReference_(path_to_soundfile, True)
         sound.play()
 
+    elif app.config.get('soundplayer') == '':
+        try:
+            import wave
+            import ossaudiodev
+        except Exception:
+            log.exception('Sound Playback Error')
+            return
+
+        def _oss_play():
+            sndfile = wave.open(path_to_soundfile, 'rb')
+            nc, sw, fr, nf, _comptype, _compname = sndfile.getparams()
+            dev = ossaudiodev.open('/dev/dsp', 'w')
+            dev.setparameters(sw * 8, nc, fr)
+            dev.write(sndfile.readframes(nf))
+            sndfile.close()
+            dev.close()
+        app.thread_interface(_oss_play)
+
+    else:
+        player = app.config.get('soundplayer')
+        command = build_command(player, path_to_soundfile)
+        exec_command(command)
+
 def get_global_show():
     maxi = 0
     for account in app.connections:
@@ -1485,8 +1466,9 @@ def call_counter(func):
     return helper
 
 def get_sync_threshold(jid, archive_info):
+    cache = caps_cache.muc_caps_cache
     if archive_info is None or archive_info.sync_threshold is None:
-        if muc_caps_cache.supports(jid, 'muc#roomconfig_membersonly'):
+        if cache.supports(jid, 'muc#roomconfig_membersonly'):
             threshold = app.config.get('private_room_sync_threshold')
         else:
             threshold = app.config.get('public_room_sync_threshold')
@@ -1505,3 +1487,62 @@ def load_json(path, key=None, default=None):
     if key is None:
         return json_dict
     return json_dict.get(key, default)
+
+class AdditionalDataDict(collections.UserDict):
+    def __init__(self, initialdata=None):
+        collections.UserDict.__init__(self, initialdata)
+
+    @staticmethod
+    def _get_path_childs(full_path):
+        path_childs = [full_path]
+        if ':' in full_path:
+            path_childs = full_path.split(':')
+        return path_childs
+
+    def set_value(self, full_path, key, value):
+        path_childs = self._get_path_childs(full_path)
+        _dict = self.data
+        for path in path_childs:
+            try:
+                _dict = _dict[path]
+            except KeyError:
+                _dict[path] = {}
+                _dict = _dict[path]
+        _dict[key] = value
+
+    def get_value(self, full_path, key, default=None):
+        path_childs = self._get_path_childs(full_path)
+        _dict = self.data
+        for path in path_childs:
+            try:
+                _dict = _dict[path]
+            except KeyError:
+                return default
+        try:
+            return _dict[key]
+        except KeyError:
+            return default
+
+    def remove_value(self, full_path, key):
+        path_childs = self._get_path_childs(full_path)
+        _dict = self.data
+        for path in path_childs:
+            try:
+                _dict = _dict[path]
+            except KeyError:
+                return
+        try:
+            del _dict[key]
+        except KeyError:
+            return
+
+
+def save_roster_position(window):
+    if not app.config.get('save-roster-position'):
+        return
+    if app.is_display(Display.WAYLAND):
+        return
+    x_pos, y_pos = window.get_position()
+    log.debug('Save roster position: %s %s', x_pos, y_pos)
+    app.config.set('roster_x-position', x_pos)
+    app.config.set('roster_y-position', y_pos)
