@@ -1,52 +1,50 @@
-# -*- coding:utf-8 -*-
-## src/message_window.py
-##
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2005-2008 Travis Shirk <travis AT pobox.com>
-##                         Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006 Geobert Quach <geobert AT gmail.com>
-##                    Dimitur Kirov <dkirov AT gmail.com>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
-##                    Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2005-2008 Travis Shirk <travis AT pobox.com>
+#                         Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006 Geobert Quach <geobert AT gmail.com>
+#                    Dimitur Kirov <dkirov AT gmail.com>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
+#                    Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import time
 
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import GObject
 from gi.repository import GLib
 
 from gajim import common
-from gajim import gtkgui_helpers
-from gajim import message_control
-from gajim import dialogs
-from gajim.chat_control_base import ChatControlBase
-from gajim.chat_control import ChatControl
 from gajim.common import app
 from gajim.common.i18n import Q_
+from gajim.common.i18n import _
+
+from gajim import gtkgui_helpers
+from gajim import message_control
+from gajim.chat_control_base import ChatControlBase
+from gajim.chat_control import ChatControl
+
+from gajim.gtk.dialogs import YesNoDialog
 
 ####################
 
-class MessageWindow(object):
+class MessageWindow:
     """
     Class for windows which contain message like things; chats, groupchats, etc
     """
@@ -72,7 +70,7 @@ class MessageWindow(object):
         # If None, the window is not tied to any specific type
         self.type_ = type_
         # dict { handler id: widget}. Keeps callbacks, which
-        # lead to cylcular references
+        # lead to circular references
         self.handlers = {}
         # Don't show warning dialogs when we want to delete the window
         self.dont_warn_on_delete = False
@@ -114,7 +112,7 @@ class MessageWindow(object):
         id_ = self.window.connect('focus-in-event', self._on_window_focus)
         self.handlers[id_] = self.window
 
-        keys=['<Control>f', '<Control>g', '<Control>h', '<Control>i',
+        keys = ['<Control>f', '<Control>g', '<Control>h', '<Control>i',
                 '<Control>l', '<Control>L', '<Control><Shift>n', '<Control>u',
                 '<Control>b', '<Control>F4',
                 '<Control>w', '<Control>Page_Up', '<Control>Page_Down', '<Alt>Right',
@@ -241,7 +239,7 @@ class MessageWindow(object):
                         ctrl.minimize()
                 # destroy window
                 return False
-            dialogs.YesNoDialog(
+            YesNoDialog(
                 _('You are going to close several tabs'),
                 _('Do you really want to close them all?'),
                 checktext=_('_Do not ask me again'), on_response_yes=on_yes1,
@@ -292,16 +290,10 @@ class MessageWindow(object):
         self._controls[control.account][fjid] = control
 
         if self.get_num_controls() == 2:
-            # is first conversation_textview scrolled down ?
-            scrolled = False
             first_widget = self.notebook.get_nth_page(0)
             ctrl = self._widget_to_control(first_widget)
-            conv_textview = ctrl.conv_textview
-            if conv_textview.at_the_end():
-                scrolled = True
             self.notebook.set_show_tabs(True)
-            if scrolled:
-                GLib.idle_add(conv_textview.scroll_to_end_iter)
+            ctrl.scroll_to_end()
 
         # Add notebook page and connect up to the tab's close button
         xml = gtkgui_helpers.get_gtk_builder('message_window.ui', 'chat_tab_ebox')
@@ -385,7 +377,7 @@ class MessageWindow(object):
                 self.window.lookup_action(
                     'information-%s' % control.control_id).activate()
                 return True
-            elif keyval == Gdk.KEY_l or keyval == Gdk.KEY_L: # CTRL + l|L
+            elif keyval in (Gdk.KEY_l, Gdk.KEY_L): # CTRL + l|L
                 control.conv_textview.clear()
                 return True
             elif keyval == Gdk.KEY_u: # CTRL + u: emacs style clear line
@@ -405,7 +397,7 @@ class MessageWindow(object):
                 self.remove_tab(control, self.CLOSE_CTRL_KEY)
                 return True
             elif keyval == Gdk.KEY_w: # CTRL + w
-                # CTRL + w removes latest word before sursor when User uses emacs
+                # CTRL + w removes latest word before cursor when User uses emacs
                 # theme
                 if not Gtk.Settings.get_default().get_property(
                 'gtk-key-theme-name') == 'Emacs':
@@ -438,23 +430,28 @@ class MessageWindow(object):
                     new = 0
                 self.notebook.set_current_page(new)
                 return True
-            elif keyval == Gdk.KEY_Left: # ALT + LEFT
+
+            if keyval == Gdk.KEY_Left: # ALT + LEFT
                 new = self.notebook.get_current_page() - 1
                 if new < 0:
                     new = self.notebook.get_n_pages() - 1
                 self.notebook.set_current_page(new)
                 return True
-            elif chr(keyval) in st: # ALT + 1,2,3..
+
+            if chr(keyval) in st: # ALT + 1,2,3..
                 self.notebook.set_current_page(st.index(chr(keyval)))
                 return True
-            elif keyval == Gdk.KEY_m: # ALT + M show emoticons menu
+
+            if keyval == Gdk.KEY_m: # ALT + M show emoticons menu
                 control.emoticons_button.get_popover().show()
                 return True
-            elif control.type_id == message_control.TYPE_GC and \
-            keyval == Gdk.KEY_t: # ALT + t
+
+            if (control.type_id == message_control.TYPE_GC and
+                    keyval == Gdk.KEY_t): # ALT + t
                 self.window.lookup_action(
                     'change-subject-%s' % control.control_id).activate()
                 return True
+
         # Close tab bindings
         elif keyval == Gdk.KEY_Escape and \
         app.config.get('escape_key_closes'): # Escape
@@ -552,7 +549,7 @@ class MessageWindow(object):
         self.window.present()
         GLib.idle_add(ctrl.msg_textview.grab_focus)
 
-    def remove_tab(self, ctrl, method, reason = None, force = False):
+    def remove_tab(self, ctrl, method, reason=None, force=False):
         """
         Reason is only for gc (offline status message) if force is True, do not
         ask any confirmation
@@ -563,8 +560,9 @@ class MessageWindow(object):
             else: # We are leaving gc without status message or it's a chat
                 ctrl.shutdown()
             # Update external state
-            app.events.remove_events(ctrl.account, ctrl.get_full_jid,
-                    types = ['printed_msg', 'chat', 'gc_msg'])
+            app.events.remove_events(
+                ctrl.account, ctrl.get_full_jid,
+                types=['printed_msg', 'chat', 'gc_msg'])
 
             fjid = ctrl.get_full_jid()
             jid = app.get_jid_without_resource(fjid)
@@ -581,7 +579,7 @@ class MessageWindow(object):
 
             del self._controls[ctrl.account][fjid]
 
-            if len(self._controls[ctrl.account]) == 0:
+            if not self._controls[ctrl.account]:
                 del self._controls[ctrl.account]
 
             self.check_tabs()
@@ -650,15 +648,14 @@ class MessageWindow(object):
         if isinstance(ctrl, ChatControl):
             tab_label_str = ctrl.get_tab_label()
             # Set Label Color
-            class_name = 'state_{}_color'.format(chatstate)
-            gtkgui_helpers.add_css_class(nick_label, class_name)
+            gtkgui_helpers.add_css_class(nick_label, chatstate, 'gajim-state-')
         else:
             tab_label_str, color = ctrl.get_tab_label(chatstate)
             # Set Label Color
             if color == 'active':
-                gtkgui_helpers.add_css_class(nick_label, None)
+                gtkgui_helpers.add_css_class(nick_label, None, 'gajim-state-')
             elif color is not None:
-                gtkgui_helpers.add_css_class(nick_label, color)
+                gtkgui_helpers.add_css_class(nick_label, color, 'gajim-state-')
 
         nick_label.set_markup(tab_label_str)
 
@@ -734,7 +731,7 @@ class MessageWindow(object):
             return self._widget_to_control(nth_child)
 
     def has_control(self, jid, acct):
-        return (acct in self._controls and jid in self._controls[acct])
+        return acct in self._controls and jid in self._controls[acct]
 
     def change_key(self, old_jid, new_jid, acct):
         """
@@ -774,7 +771,7 @@ class MessageWindow(object):
         # if no others controls have awaiting events
         # loop until finding an unread tab or having done a complete cycle
         while True:
-            if forward == True: # look for the first unread tab on the right
+            if forward is True: # look for the first unread tab on the right
                 ind = ind + 1
                 if ind >= self.notebook.get_n_pages():
                     ind = 0
@@ -786,7 +783,7 @@ class MessageWindow(object):
             if ctrl.get_nb_unread() > 0:
                 found = True
                 break # found
-            elif app.config.get('ctrl_tab_go_to_next_composing') :
+            elif app.config.get('ctrl_tab_go_to_next_composing'):
                 # Search for a composing contact
                 contact = ctrl.contact
                 if first_composing_ind == -1 and contact.chatstate == 'composing':
@@ -828,7 +825,7 @@ class MessageWindow(object):
 
         new_ctrl = self._widget_to_control(notebook.get_nth_page(page_num))
         new_ctrl.set_control_active(True)
-        self.show_title(control = new_ctrl)
+        self.show_title(control=new_ctrl)
 
         control = self.get_active_control()
         if isinstance(control, ChatControlBase):
@@ -849,20 +846,22 @@ class MessageWindow(object):
                 self.move_to_next_unread_tab(False)
                 return True
             # SHIFT + PAGE_[UP|DOWN]: send to conv_textview
-            elif event.keyval in (Gdk.KEY_Page_Down, Gdk.KEY_Page_Up):
+            if event.keyval in (Gdk.KEY_Page_Down, Gdk.KEY_Page_Up):
                 control.conv_textview.tv.event(event)
                 return True
+
         elif event.get_state() & Gdk.ModifierType.CONTROL_MASK:
             if event.keyval == Gdk.KEY_Tab: # CTRL + TAB
                 self.move_to_next_unread_tab(True)
                 return True
             # Ctrl+PageUP / DOWN has to be handled by notebook
-            elif event.keyval == Gdk.KEY_Page_Down:
+            if event.keyval == Gdk.KEY_Page_Down:
                 self.move_to_next_unread_tab(True)
                 return True
-            elif event.keyval == Gdk.KEY_Page_Up:
+            if event.keyval == Gdk.KEY_Page_Up:
                 self.move_to_next_unread_tab(False)
                 return True
+
         if event.keyval in (Gdk.KEY_Shift_L, Gdk.KEY_Shift_R,
         Gdk.KEY_Control_L, Gdk.KEY_Control_R, Gdk.KEY_Caps_Lock,
         Gdk.KEY_Shift_Lock, Gdk.KEY_Meta_L, Gdk.KEY_Meta_R,
@@ -1049,22 +1048,26 @@ class MessageWindowMgr(GObject.GObject):
 
         gtkgui_helpers.move_window(win.window, pos[0], pos[1])
 
-    def _mode_to_key(self, contact, acct, type_, resource = None):
+    def _mode_to_key(self, contact, acct, type_, resource=None):
         if self.mode == self.ONE_MSG_WINDOW_NEVER:
             key = acct + contact.jid
             if resource:
                 key += '/' + resource
             return key
-        elif self.mode == self.ONE_MSG_WINDOW_ALWAYS:
+
+        if self.mode == self.ONE_MSG_WINDOW_ALWAYS:
             return self.MAIN_WIN
-        elif self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+
+        if self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
             return self.ROSTER_MAIN_WIN
-        elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
+
+        if self.mode == self.ONE_MSG_WINDOW_PERACCT:
             return acct
-        elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
+
+        if self.mode == self.ONE_MSG_WINDOW_PERTYPE:
             return type_
 
-    def create_window(self, contact, acct, type_, resource = None):
+    def create_window(self, contact, acct, type_, resource=None):
         win_acct = None
         win_type = None
         win_role = None # X11 window role
@@ -1119,7 +1122,7 @@ class MessageWindowMgr(GObject.GObject):
         return False
 
     def _on_window_destroy(self, win):
-        for k in self._windows.keys():
+        for k in list(self._windows.keys()):
             if self._windows[k].window == win:
                 self.emit('window-delete', self._windows[k])
                 del self._windows[k]
@@ -1207,7 +1210,7 @@ class MessageWindowMgr(GObject.GObject):
         if x < 0 or y < 0 or width < 0 or height < 0:
             return
 
-        elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
+        if self.mode == self.ONE_MSG_WINDOW_PERACCT:
             acct = msg_win.account
         elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
             type_ = msg_win.type_
