@@ -406,9 +406,9 @@ class PluginManager(metaclass=Singleton):
             return
         for con in app.connections.values():
             for module in plugin.modules:
-                instance, name = module.get_instance(con)
                 if not module.zeroconf and con.name == 'Local':
                     continue
+                instance, name = module.get_instance(con)
                 modules.register_single(con, instance, name)
 
                 # If handlers have been registered, register the
@@ -558,9 +558,9 @@ class PluginManager(metaclass=Singleton):
                 return
 
             for module in plugin.modules:
-                instance, name = module.get_instance(con)
                 if not module.zeroconf and con.name == 'Local':
                     continue
+                instance, name = module.get_instance(con)
                 modules.register_single(con, instance, name)
 
     def _plugin_is_active_in_global_config(self, plugin):
@@ -650,6 +650,10 @@ class PluginManager(metaclass=Singleton):
             min_v = conf.get('info', 'min_gajim_version', fallback=None)
             max_v = conf.get('info', 'max_gajim_version', fallback=None)
 
+            if min_v is None or max_v is None:
+                log.warning('Plugin without min/max version: %s', elem_name)
+                continue
+
             gajim_v = gajim.__version__.split('+', 1)[0]
             gajim_v_cmp = parse_version(gajim_v)
 
