@@ -1,85 +1,56 @@
-# -*- coding:utf-8 -*-
-## src/vcard.py
-##
-## Copyright (C) 2003-2017 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006 Junglecow J <junglecow AT gmail.com>
-##                    Dimitur Kirov <dkirov AT gmail.com>
-##                    Travis Shirk <travis AT pobox.com>
-##                    Stefan Bethge <stefan AT lanpartei.de>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
-## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##                    Stephan Erb <steve-e AT h3c.de>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
+# Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006 Junglecow J <junglecow AT gmail.com>
+#                    Dimitur Kirov <dkirov AT gmail.com>
+#                    Travis Shirk <travis AT pobox.com>
+#                    Stefan Bethge <stefan AT lanpartei.de>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+# Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#                    Stephan Erb <steve-e AT h3c.de>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-# THIS FILE IS FOR **OTHERS'** PROFILE (when we VIEW their INFO)
-
-import gtk
-import gobject
-import base64
-import time
-import locale
 import os
+import base64
+import binascii
 
-import gtkgui_helpers
+from gi.repository import Gtk
+from gi.repository import GLib
+from gi.repository import Gdk
 
-from common import helpers
-from common import gajim
-from common import ged
-from common.i18n import Q_
+from gajim import gtkgui_helpers
+from gajim.common import helpers
+from gajim.common import app
+from gajim.common import ged
+from gajim.common import configpaths
+from gajim.common.i18n import Q_
+from gajim.common.i18n import _
+from gajim.common.const import AvatarSize
 
-def get_avatar_pixbuf_encoded_mime(photo):
-    """
-    Return the pixbuf of the image
+# log = logging.getLogger('gajim.vcard')
 
-    Photo is a dictionary containing PHOTO information.
-    """
-    if not isinstance(photo, dict):
-        return None, None, None
-    img_decoded = None
-    avatar_encoded = None
-    avatar_mime_type = None
-    if 'BINVAL' in photo:
-        img_encoded = photo['BINVAL']
-        avatar_encoded = img_encoded
-        try:
-            img_decoded = base64.decodestring(img_encoded)
-        except Exception:
-            pass
-    if img_decoded:
-        if 'TYPE' in photo:
-            avatar_mime_type = photo['TYPE']
-            pixbuf = gtkgui_helpers.get_pixbuf_from_data(img_decoded)
-        else:
-            pixbuf, avatar_mime_type = gtkgui_helpers.get_pixbuf_from_data(
-                                            img_decoded, want_type=True)
-    else:
-        pixbuf = None
-    return pixbuf, avatar_encoded, avatar_mime_type
 
 class VcardWindow:
     """
     Class for contact's information window
     """
 
-    def __init__(self, contact, account, gc_contact = None):
+    def __init__(self, contact, account, gc_contact=None):
         # the contact variable is the jid if vcard is true
         self.xml = gtkgui_helpers.get_gtk_builder('vcard_information_window.ui')
         self.window = self.xml.get_object('vcard_information_window')
@@ -88,11 +59,12 @@ class VcardWindow:
         self.contact = contact
         self.account = account
         self.gc_contact = gc_contact
+        self.avatar = None
 
         # Get real jid
         if gc_contact:
             # Don't use real jid if room is (semi-)anonymous
-            gc_control = gajim.interface.msg_win_mgr.get_gc_control(
+            gc_control = app.interface.msg_win_mgr.get_gc_control(
             gc_contact.room_jid, account)
             if gc_contact.jid and not gc_control.is_anonymous:
                 self.real_jid = gc_contact.jid
@@ -108,7 +80,7 @@ class VcardWindow:
             self.real_resource = contact.resource
 
         puny_jid = helpers.sanitize_filename(contact.jid)
-        local_avatar_basepath = os.path.join(gajim.AVATAR_PATH, puny_jid) + \
+        local_avatar_basepath = os.path.join(configpaths.get('AVATAR'), puny_jid) + \
                 '_local'
         for extension in ('.png', '.jpeg'):
             local_avatar_path = local_avatar_basepath + extension
@@ -118,90 +90,109 @@ class VcardWindow:
                 image.show()
                 self.xml.get_object('custom_avatar_label').show()
                 break
-        self.avatar_mime_type = None
-        self.avatar_encoded = None
         self.vcard_arrived = False
         self.os_info_arrived = False
         self.entity_time_arrived = False
-        self.update_progressbar_timeout_id = gobject.timeout_add(100,
-                self.update_progressbar)
+        self.time = 0
+        self.update_intervall = 100  # Milliseconds
+        self.update_progressbar_timeout_id = GLib.timeout_add(self.update_intervall,
+            self.update_progressbar)
 
-        gajim.ged.register_event_handler('version-result-received', ged.GUI1,
+        app.ged.register_event_handler('version-result-received', ged.GUI1,
             self.set_os_info)
-        gajim.ged.register_event_handler('last-result-received', ged.GUI2,
-            self.set_last_status_time)
-        gajim.ged.register_event_handler('time-result-received', ged.GUI1,
+        app.ged.register_event_handler('time-result-received', ged.GUI1,
             self.set_entity_time)
-        gajim.ged.register_event_handler('vcard-received', ged.GUI1,
-            self._nec_vcard_received)
 
         self.fill_jabber_page()
-        annotations = gajim.connections[self.account].annotations
+        con = app.connections[self.account]
+        annotations = con.get_module('Annotations').annotations
         if self.contact.jid in annotations:
             buffer_ = self.xml.get_object('textview_annotation').get_buffer()
             buffer_.set_text(annotations[self.contact.jid])
 
+        for widget_name in ('URL_label',
+                            'EMAIL_WORK_USERID_label',
+                            'EMAIL_HOME_USERID_label'):
+            widget = self.xml.get_object(widget_name)
+            widget.hide()
+
         self.xml.connect_signals(self)
         self.xml.get_object('close_button').grab_focus()
         self.window.show_all()
 
     def update_progressbar(self):
         self.progressbar.pulse()
-        return True # loop forever
+        self.time += self.update_intervall
+        # Timeout in Milliseconds
+        if (self.vcard_arrived and self.os_info_arrived and
+                self.entity_time_arrived) or self.time == 10000:
+            self.progressbar.hide()
+            self.update_progressbar_timeout_id = None
+            return False
+        return True
 
     def on_vcard_information_window_destroy(self, widget):
         if self.update_progressbar_timeout_id is not None:
-            gobject.source_remove(self.update_progressbar_timeout_id)
-        del gajim.interface.instances[self.account]['infos'][self.contact.jid]
+            GLib.source_remove(self.update_progressbar_timeout_id)
+        del app.interface.instances[self.account]['infos'][self.contact.jid]
         buffer_ = self.xml.get_object('textview_annotation').get_buffer()
-        annotation = buffer_.get_text(buffer_.get_start_iter(),
-                buffer_.get_end_iter())
-        connection = gajim.connections[self.account]
-        if annotation != connection.annotations.get(self.contact.jid, ''):
-            connection.annotations[self.contact.jid] = annotation
-            connection.store_annotations()
-        gajim.ged.remove_event_handler('version-result-received', ged.GUI1,
+        new_annotation = buffer_.get_text(buffer_.get_start_iter(),
+                buffer_.get_end_iter(), True)
+        con = app.connections[self.account]
+        annotations = con.get_module('Annotations').annotations
+        if new_annotation != annotations.get(self.contact.jid, ''):
+            annotations[self.contact.jid] = new_annotation
+            con.get_module('Annotations').store_annotations()
+        app.ged.remove_event_handler('version-result-received', ged.GUI1,
             self.set_os_info)
-        gajim.ged.remove_event_handler('last-result-received', ged.GUI2,
-            self.set_last_status_time)
-        gajim.ged.remove_event_handler('time-result-received', ged.GUI1,
+        app.ged.remove_event_handler('time-result-received', ged.GUI1,
             self.set_entity_time)
-        gajim.ged.remove_event_handler('vcard-received', ged.GUI1,
-            self._nec_vcard_received)
 
     def on_vcard_information_window_key_press_event(self, widget, event):
-        if event.keyval == gtk.keysyms.Escape:
+        if event.keyval == Gdk.KEY_Escape:
             self.window.destroy()
 
     def on_information_notebook_switch_page(self, widget, page, page_num):
-        gobject.idle_add(self.xml.get_object('close_button').grab_focus)
+        GLib.idle_add(self.xml.get_object('close_button').grab_focus)
 
     def on_PHOTO_eventbox_button_press_event(self, widget, event):
         """
         If right-clicked, show popup
         """
         if event.button == 3: # right click
-            menu = gtk.Menu()
-            menuitem = gtk.ImageMenuItem(gtk.STOCK_SAVE_AS)
+            menu = Gtk.Menu()
+            menuitem = Gtk.MenuItem.new_with_mnemonic(_('Save _As'))
+            if self.gc_contact:
+                sha = self.gc_contact.avatar_sha
+                name = self.gc_contact.get_shown_name()
+            else:
+                sha = app.contacts.get_avatar_sha(
+                    self.account, self.contact.jid)
+                name = self.contact.get_shown_name()
+            if sha is None:
+                sha = self.avatar
             menuitem.connect('activate',
-                    gtkgui_helpers.on_avatar_save_as_menuitem_activate,
-                    self.contact.jid, self.contact.get_shown_name())
+                gtkgui_helpers.on_avatar_save_as_menuitem_activate, sha, name)
             menu.append(menuitem)
-            menu.connect('selection-done', lambda w:w.destroy())
+            menu.connect('selection-done', lambda w: w.destroy())
             # show the menu
             menu.show_all()
-            menu.popup(None, None, None, event.button, event.time)
+            menu.attach_to_widget(widget, None)
+            menu.popup(None, None, None, None, event.button, event.time)
 
     def set_value(self, entry_name, value):
         try:
-            if value and entry_name == 'URL_label':
-                widget = gtk.LinkButton(value, value)
-                widget.set_alignment(0, 0)
-                widget.show()
-                table = self.xml.get_object('personal_info_table')
-                table.attach(widget, 1, 4, 3, 4, yoptions = 0)
+            widget = self.xml.get_object(entry_name)
+            if entry_name in ('URL_label',
+                              'EMAIL_WORK_USERID_label',
+                              'EMAIL_HOME_USERID_label'):
+                if entry_name == 'URL_label':
+                    widget.set_uri(value)
+                else:
+                    widget.set_uri('mailto:' + value)
+                widget.set_label(value)
+                self.xml.get_object(entry_name).show()
             else:
-                widget = self.xml.get_object(entry_name)
                 val = widget.get_text()
                 if val:
                     value = val + ' / ' + value
@@ -209,21 +200,33 @@ class VcardWindow:
         except AttributeError:
             pass
 
-    def set_values(self, vcard):
+    def _set_values(self, vcard, jid):
         for i in vcard.keys():
             if i == 'PHOTO' and self.xml.get_object('information_notebook').\
             get_n_pages() > 4:
-                pixbuf, self.avatar_encoded, self.avatar_mime_type = \
-                        get_avatar_pixbuf_encoded_mime(vcard[i])
+                if 'BINVAL' not in vcard[i]:
+                    continue
+                photo_encoded = vcard[i]['BINVAL']
+                if photo_encoded == '':
+                    continue
+                try:
+                    photo_decoded = base64.b64decode(
+                        photo_encoded.encode('utf-8'))
+                except binascii.Error as error:
+                    app.log('avatar').warning('Invalid avatar for %s: %s', jid, error)
+                    continue
+
+                pixbuf = gtkgui_helpers.get_pixbuf_from_data(photo_decoded)
+                if pixbuf is None:
+                    continue
+                self.avatar = pixbuf
+                pixbuf = gtkgui_helpers.scale_pixbuf(pixbuf, AvatarSize.VCARD)
+                surface = Gdk.cairo_surface_create_from_pixbuf(
+                    pixbuf, self.window.get_scale_factor())
                 image = self.xml.get_object('PHOTO_image')
+                image.set_from_surface(surface)
                 image.show()
                 self.xml.get_object('user_avatar_label').show()
-                if not pixbuf:
-                    image.set_from_icon_name('stock_person',
-                            gtk.ICON_SIZE_DIALOG)
-                    continue
-                pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf, 'vcard')
-                image.set_from_pixbuf(pixbuf)
                 continue
             if i in ('ADR', 'TEL', 'EMAIL'):
                 for entry in vcard[i]:
@@ -238,19 +241,21 @@ class VcardWindow:
             else:
                 if i == 'DESC':
                     self.xml.get_object('DESC_textview').get_buffer().set_text(
-                            vcard[i], 0)
+                        vcard[i], len(vcard[i].encode('utf-8')))
                 elif i != 'jid': # Do not override jid_label
                     self.set_value(i + '_label', vcard[i])
         self.vcard_arrived = True
-        self.test_remove_progressbar()
 
     def clear_values(self):
         for l in ('FN', 'NICKNAME', 'N_FAMILY', 'N_GIVEN', 'N_MIDDLE',
         'N_PREFIX', 'N_SUFFIX', 'EMAIL_HOME_USERID', 'TEL_HOME_NUMBER', 'BDAY',
         'ORG_ORGNAME', 'ORG_ORGUNIT', 'TITLE', 'ROLE', 'EMAIL_WORK_USERID',
-        'TEL_WORK_NUMBER'):
+        'TEL_WORK_NUMBER', 'URL'):
             widget = self.xml.get_object(l + '_label')
-            widget.set_text('')
+            if l in ('EMAIL_HOME_USERID', 'EMAIL_WORK_USERID', 'URL'):
+                widget.hide()
+            else:
+                widget.set_text('')
         for pref in ('ADR_HOME', 'ADR_WORK'):
             for l in ('STREET', 'EXTADR', 'LOCALITY', 'PCODE', 'REGION',
             'CTRY'):
@@ -258,93 +263,82 @@ class VcardWindow:
                 widget.set_text('')
         self.xml.get_object('DESC_textview').get_buffer().set_text('')
 
-
-    def _nec_vcard_received(self, obj):
-        if obj.conn.name != self.account:
-            return
-        if obj.resource:
-            # It's a muc occupant vcard
-            if obj.fjid != self.contact.jid:
-                return
-        else:
-            if obj.jid != self.contact.jid:
-                return
+    def _nec_vcard_received(self, jid, resource, room, vcard, *args):
         self.clear_values()
-        self.set_values(obj.vcard_dict)
-
-    def test_remove_progressbar(self):
-        if self.update_progressbar_timeout_id is not None and \
-        self.vcard_arrived and self.os_info_arrived and self.entity_time_arrived:
-            gobject.source_remove(self.update_progressbar_timeout_id)
-            self.progressbar.hide()
-            self.update_progressbar_timeout_id = None
-
-    def set_last_status_time(self, obj):
-        if obj.conn.name != self.account:
-            return
-        if obj.fjid != self.real_jid:
-            return
-        self.fill_status_label()
+        self._set_values(vcard, jid)
 
     def set_os_info(self, obj):
         if obj.conn.name != self.account:
             return
         if self.xml.get_object('information_notebook').get_n_pages() < 5:
             return
-        if obj.fjid != self.contact.jid and obj.fjid != self.real_jid:
+        if self.gc_contact:
+            if obj.jid != self.contact.jid:
+                return
+        elif obj.jid.getStripped() != self.contact.jid:
             return
         i = 0
         client = ''
-        os = ''
+        os_info = ''
         while i in self.os_info:
-            if not self.os_info[i]['resource'] or \
-            self.os_info[i]['resource'] == obj.resource:
-                self.os_info[i]['client'] = obj.client_info
-                self.os_info[i]['os'] = obj.os_info
+            if self.os_info[i]['resource'] == obj.jid.getResource():
+                if obj.client_info:
+                    self.os_info[i]['client'] = obj.client_info
+                else:
+                    self.os_info[i]['client'] = Q_('?Client:Unknown')
+                if obj.os_info:
+                    self.os_info[i]['os'] = obj.os_info
+                else:
+                    self.os_info[i]['os'] = Q_('?OS:Unknown')
+            else:
+                if not self.os_info[i]['client']:
+                    self.os_info[i]['client'] = Q_('?Client:Unknown')
+                if not self.os_info[i]['os']:
+                    self.os_info[i]['os'] = Q_('?OS:Unknown')
             if i > 0:
                 client += '\n'
-                os += '\n'
+                os_info += '\n'
             client += self.os_info[i]['client']
-            os += self.os_info[i]['os']
+            os_info += self.os_info[i]['os']
             i += 1
 
-        if client == '':
-            client = Q_('?Client:Unknown')
-        if os == '':
-            os = Q_('?OS:Unknown')
         self.xml.get_object('client_name_version_label').set_text(client)
-        self.xml.get_object('os_label').set_text(os)
+        self.xml.get_object('os_label').set_text(os_info)
         self.os_info_arrived = True
-        self.test_remove_progressbar()
 
     def set_entity_time(self, obj):
         if obj.conn.name != self.account:
             return
         if self.xml.get_object('information_notebook').get_n_pages() < 5:
             return
-        if obj.fjid != self.contact.jid and obj.fjid != self.real_jid:
+        if self.gc_contact:
+            if obj.jid != self.contact.jid:
+                return
+        elif obj.jid.getStripped() != self.contact.jid:
             return
         i = 0
         time_s = ''
         while i in self.time_info:
-            if not self.time_info[i]['resource'] or \
-            self.time_info[i]['resource'] == obj.resource:
-                self.time_info[i]['time'] = obj.time_info
+            if self.time_info[i]['resource'] == obj.jid.getResource():
+                if obj.time_info:
+                    self.time_info[i]['time'] = obj.time_info
+                else:
+                    self.time_info[i]['time'] = Q_('?Time:Unknown')
+            else:
+                if not self.time_info[i]['time']:
+                    self.time_info[i]['time'] = Q_('?Time:Unknown')
             if i > 0:
                 time_s += '\n'
             time_s += self.time_info[i]['time']
             i += 1
 
-        if time_s == '':
-            time_s = Q_('?Time:Unknown')
         self.xml.get_object('time_label').set_text(time_s)
         self.entity_time_arrived = True
-        self.test_remove_progressbar()
 
     def fill_status_label(self):
         if self.xml.get_object('information_notebook').get_n_pages() < 5:
             return
-        contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
+        contact_list = app.contacts.get_contacts(self.account, self.contact.jid)
         connected_contact_list = []
         for c in contact_list:
             if c.show not in ('offline', 'error'):
@@ -359,29 +353,19 @@ class VcardWindow:
             stats = helpers.get_uf_show(self.contact.show)
             if self.contact.status:
                 stats += ': ' + self.contact.status
-            if self.contact.last_status_time:
-                stats += '\n' + _('since %s') % time.strftime('%c',
-                        self.contact.last_status_time).decode(
-                        locale.getpreferredencoding())
             for c in connected_contact_list:
                 if c.resource != self.contact.resource:
                     stats += '\n'
                     stats += helpers.get_uf_show(c.show)
                     if c.status:
                         stats += ': ' + c.status
-                    if c.last_status_time:
-                        stats += '\n' + _('since %s') % time.strftime('%c',
-                                c.last_status_time).decode(locale.getpreferredencoding())
         else: # Maybe gc_vcard ?
             stats = helpers.get_uf_show(self.contact.show)
             if self.contact.status:
                 stats += ': ' + self.contact.status
         status_label = self.xml.get_object('status_label')
-        status_label.set_max_width_chars(15)
         status_label.set_text(stats)
-
-        status_label_eventbox = self.xml.get_object('status_label_eventbox')
-        status_label_eventbox.set_tooltip_text(stats)
+        status_label.set_tooltip_text(stats)
 
     def fill_jabber_page(self):
         self.xml.get_object('nickname_label').set_markup(
@@ -403,42 +387,32 @@ class VcardWindow:
         else:
             uf_sub = helpers.get_uf_sub(self.contact.sub)
             subscription_label.set_text(uf_sub)
-            eb = self.xml.get_object('subscription_label_eventbox')
             if self.contact.sub == 'from':
-                tt_text = _("This contact is interested in your presence information, but you are not interested in his/her presence")
+                tt_text = _("This contact is interested in your presence information, but you are not interested in their presence")
             elif self.contact.sub == 'to':
-                tt_text = _("You are interested in the contact's presence information, but he/she is not interested in yours")
+                tt_text = _("You are interested in the contact's presence information, but it is not mutual")
             elif self.contact.sub == 'both':
-                tt_text = _("You and the contact are interested in each other's presence information")
+                tt_text = _("The contact and you want to exchange presence information")
             else: # None
-                tt_text = _("You are not interested in the contact's presence, and neither he/she is interested in yours")
-            eb.set_tooltip_text(tt_text)
+                tt_text = _("You and the contact have a mutual disinterest in each-others presence information")
+            subscription_label.set_tooltip_text(tt_text)
 
             uf_ask = helpers.get_uf_ask(self.contact.ask)
             ask_label.set_text(uf_ask)
-            eb = self.xml.get_object('ask_label_eventbox')
             if self.contact.ask == 'subscribe':
                 tt_text = _("You are waiting contact's answer about your subscription request")
             else:
                 tt_text = _("There is no pending subscription request.")
-            eb.set_tooltip_text(tt_text)
+            ask_label.set_tooltip_text(tt_text)
 
-        resources = '%s (%s)' % (self.contact.resource, unicode(
-                self.contact.priority))
+        resources = '%s (%s)' % (self.contact.resource, str(
+            self.contact.priority))
         uf_resources = self.contact.resource + _(' resource with priority ')\
-                + unicode(self.contact.priority)
+                + str(self.contact.priority)
         if not self.contact.status:
             self.contact.status = ''
 
-        # Request list time status only if contact is offline
-        if self.contact.show == 'offline':
-            if self.gc_contact:
-                j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
-                gajim.connections[self.account].request_last_status_time(j, r,
-                        self.contact.jid)
-            else:
-                gajim.connections[self.account].request_last_status_time(
-                        self.contact.jid, '')
+        con = app.connections[self.account]
 
         # do not wait for os_info if contact is not connected or has error
         # additional check for observer is needed, as show is offline for him
@@ -447,12 +421,12 @@ class VcardWindow:
             self.os_info_arrived = True
         else: # Request os info if contact is connected
             if self.gc_contact:
-                j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
-                gobject.idle_add(gajim.connections[self.account].request_os_info,
-                        j, r, self.contact.jid)
+                j, r = app.get_room_and_nick_from_fjid(self.real_jid)
+                GLib.idle_add(con.get_module('SoftwareVersion').request_os_info,
+                              j, r)
             else:
-                gobject.idle_add(gajim.connections[self.account].request_os_info,
-                        self.contact.jid, self.contact.resource)
+                GLib.idle_add(con.get_module('SoftwareVersion').request_os_info,
+                              self.contact.jid, self.contact.resource)
 
         # do not wait for entity_time if contact is not connected or has error
         # additional check for observer is needed, as show is offline for him
@@ -461,31 +435,30 @@ class VcardWindow:
             self.entity_time_arrived = True
         else: # Request entity time if contact is connected
             if self.gc_contact:
-                j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
-                gobject.idle_add(gajim.connections[self.account].\
-                        request_entity_time, j, r, self.contact.jid)
+                j, r = app.get_room_and_nick_from_fjid(self.real_jid)
+                GLib.idle_add(con.get_module('EntityTime').request_entity_time,
+                              j, r)
             else:
-                gobject.idle_add(gajim.connections[self.account].\
-                        request_entity_time, self.contact.jid, self.contact.resource)
+                GLib.idle_add(con.get_module('EntityTime').request_entity_time,
+                              self.contact.jid, self.contact.resource)
 
         self.os_info = {0: {'resource': self.real_resource, 'client': '',
                 'os': ''}}
         self.time_info = {0: {'resource': self.real_resource, 'time': ''}}
         i = 1
-        contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
+        contact_list = app.contacts.get_contacts(self.account, self.contact.jid)
         if contact_list:
             for c in contact_list:
                 if c.resource != self.contact.resource:
                     resources += '\n%s (%s)' % (c.resource,
-                            unicode(c.priority))
+                            str(c.priority))
                     uf_resources += '\n' + c.resource + \
-                            _(' resource with priority ') + unicode(c.priority)
+                            _(' resource with priority ') + str(c.priority)
                     if c.show not in ('offline', 'error'):
-                        gobject.idle_add(
-                                gajim.connections[self.account].request_os_info, c.jid,
-                                c.resource)
-                        gobject.idle_add(gajim.connections[self.account].\
-                                request_entity_time, c.jid, c.resource)
+                        GLib.idle_add(con.get_module('SoftwareVersion').request_os_info,
+                                      c.jid, c.resource)
+                        GLib.idle_add(con.get_module('EntityTime').request_entity_time,
+                                      c.jid, c.resource)
                     self.os_info[i] = {'resource': c.resource, 'client': '',
                             'os': ''}
                     self.time_info[i] = {'resource': c.resource, 'time': ''}
@@ -499,18 +472,20 @@ class VcardWindow:
         self.fill_status_label()
 
         if self.gc_contact:
-            # If we know the real jid, remove the resource from vcard request
-            gajim.connections[self.account].request_vcard(self.real_jid_for_vcard,
-                    self.gc_contact.get_full_jid())
+            con.get_module('VCardTemp').request_vcard(
+                self._nec_vcard_received,
+                self.gc_contact.get_full_jid(),
+                room=True)
         else:
-            gajim.connections[self.account].request_vcard(self.contact.jid)
+            con.get_module('VCardTemp').request_vcard(
+                self._nec_vcard_received, self.contact.jid)
 
     def on_close_button_clicked(self, widget):
         self.window.destroy()
 
 
 class ZeroconfVcardWindow:
-    def __init__(self, contact, account, is_fake = False):
+    def __init__(self, contact, account, is_fake=False):
         # the contact variable is the jid if vcard is true
         self.xml = gtkgui_helpers.get_gtk_builder('zeroconf_information_window.ui')
         self.window = self.xml.get_object('zeroconf_information_window')
@@ -519,9 +494,6 @@ class ZeroconfVcardWindow:
         self.account = account
         self.is_fake = is_fake
 
-    #       self.avatar_mime_type = None
-    #       self.avatar_encoded = None
-
         self.fill_contact_page()
         self.fill_personal_page()
 
@@ -529,10 +501,10 @@ class ZeroconfVcardWindow:
         self.window.show_all()
 
     def on_zeroconf_information_window_destroy(self, widget):
-        del gajim.interface.instances[self.account]['infos'][self.contact.jid]
+        del app.interface.instances[self.account]['infos'][self.contact.jid]
 
     def on_zeroconf_information_window_key_press_event(self, widget, event):
-        if event.keyval == gtk.keysyms.Escape:
+        if event.keyval == Gdk.KEY_Escape:
             self.window.destroy()
 
     def on_PHOTO_eventbox_button_press_event(self, widget, event):
@@ -540,24 +512,25 @@ class ZeroconfVcardWindow:
         If right-clicked, show popup
         """
         if event.button == 3: # right click
-            menu = gtk.Menu()
-            menuitem = gtk.ImageMenuItem(gtk.STOCK_SAVE_AS)
+            menu = Gtk.Menu()
+            menuitem = Gtk.MenuItem.new_with_mnemonic(_('Save _As'))
             menuitem.connect('activate',
                     gtkgui_helpers.on_avatar_save_as_menuitem_activate,
-                    self.contact.jid, self.contact.get_shown_name())
+                    self.contact.avatar_sha, self.contact.get_shown_name())
             menu.append(menuitem)
-            menu.connect('selection-done', lambda w:w.destroy())
+            menu.connect('selection-done', lambda w: w.destroy())
             # show the menu
             menu.show_all()
-            menu.popup(None, None, None, event.button, event.time)
+            menu.attach_to_widget(widget, None)
+            menu.popup(None, None, None, None, event.button, event.time)
 
     def set_value(self, entry_name, value):
         try:
             if value and entry_name == 'URL_label':
-                widget = gtk.LinkButton(value, value)
+                widget = Gtk.LinkButton(uri=value, label=value)
                 widget.set_alignment(0, 0)
                 table = self.xml.get_object('personal_info_table')
-                table.attach(widget, 1, 4, 3, 4, yoptions = 0)
+                table.attach(widget, 1, 3, 2, 1)
             else:
                 self.xml.get_object(entry_name).set_text(value)
         except AttributeError:
@@ -566,7 +539,7 @@ class ZeroconfVcardWindow:
     def fill_status_label(self):
         if self.xml.get_object('information_notebook').get_n_pages() < 2:
             return
-        contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
+        contact_list = app.contacts.get_contacts(self.account, self.contact.jid)
         # stats holds show and status message
         stats = ''
         one = True # Are we adding the first line ?
@@ -577,20 +550,14 @@ class ZeroconfVcardWindow:
                 stats += helpers.get_uf_show(c.show)
                 if c.status:
                     stats += ': ' + c.status
-                if c.last_status_time:
-                    stats += '\n' + _('since %s') % time.strftime('%c',
-                            c.last_status_time).decode(locale.getpreferredencoding())
                 one = False
         else: # Maybe gc_vcard ?
             stats = helpers.get_uf_show(self.contact.show)
             if self.contact.status:
                 stats += ': ' + self.contact.status
         status_label = self.xml.get_object('status_label')
-        status_label.set_max_width_chars(15)
         status_label.set_text(stats)
-
-        status_label_eventbox = self.xml.get_object('status_label_eventbox')
-        status_label_eventbox.set_tooltip_text(stats)
+        status_label.set_tooltip_text(stats)
 
     def fill_contact_page(self):
         self.xml.get_object('nickname_label').set_markup(
@@ -599,10 +566,10 @@ class ZeroconfVcardWindow:
                 '</span></b>')
         self.xml.get_object('local_jid_label').set_text(self.contact.jid)
 
-        resources = '%s (%s)' % (self.contact.resource, unicode(
-                self.contact.priority))
+        resources = '%s (%s)' % (self.contact.resource, str(
+            self.contact.priority))
         uf_resources = self.contact.resource + _(' resource with priority ')\
-                + unicode(self.contact.priority)
+                + str(self.contact.priority)
         if not self.contact.status:
             self.contact.status = ''
 
@@ -614,7 +581,7 @@ class ZeroconfVcardWindow:
         self.fill_status_label()
 
     def fill_personal_page(self):
-        contact = gajim.connections[gajim.ZEROCONF_ACC_NAME].roster.getItem(self.contact.jid)
+        contact = app.connections[app.ZEROCONF_ACC_NAME].roster.getItem(self.contact.jid)
         for key in ('1st', 'last', 'jid', 'email'):
             if key not in contact['txt_dict']:
                 contact['txt_dict'][key] = ''
